%   Motion estimation in ultrasound images using sparse and spatial
%   regularizations 
%
%
%   ----- solve the optimization problem
%
%   min   d(I,U) + lambda_s*||GU||_2+ lambda_d*||PU - Da||_2  
%   U,a                                    
%                s.t ||a_p||_0  < nbr_max_atoms  for all p
%
%   U  horizontal and vertical motions
%   a  sparse codes 
%   I - images
%   D - dictionary 
%   G - spatial gradient 
%   P - patch extracting operator 
%   d - data term 
%
%
%  -----
% Parts of this code have been borrowed from the SPAMS toolbox 
% sparsity toolbox 
% the Medical Image Registration Toolbox (MIRT), http://www.bme.ogi.edu/~myron/matlab/MIRT/

%% load data 
    clear all; 
    addpath(genpath(pwd)); 
    load('demo_data6');

%% Parameters 
    settings.lambda_d_values  = logspace(-3,1,5);                          % sparse regularization parameter
    settings.lambda_s  = 0.75;                                             % spatial regularization parameter
    i1 = 4;                                                                % first image
    I = images(:,:,i1:i1+1);                                               % pair of image I
    [settings, I]= patch_struct(I, settings);                              % patch structure 
    settings.plot_motion = 0;                                              % plot estimated motions 
    
%% Initialization 
    U = zeros(size(I));
    a = zeros(size(D,2),settings.patch_nbr,2);         
    outer_iter    = 1;                                                       
    inner_c(1)    = 100; 
    I(I==0) = nan;     
    
%% Motion estimation     
tic; 
while outer_iter<=length(settings.lambda_d_values) %&& (outer_e >= settings.outer_emax)
%=========Adjust parameters==================
settings.lambda_d = settings.lambda_d_values(outer_iter);
inner_iter = 1;   
inner_e    = 100;
disp('_____________________________________________')
disp(['outer_iter(' num2str(outer_iter) ')  -> [lambda_s:' num2str(settings.lambda_s) ' lambda_d:' num2str(settings.lambda_d)]);
  
     while inner_iter < settings.inner_itermax && inner_e >= settings.inner_emax
     %========Sparse coding=======================
     P = im2patch_2(U, settings); 
     a(:,:,1)  = mexOMP(P(:,:,1),D(:,:,1),settings);                                
     a(:,:,2)  = mexOMP(P(:,:,2),D(:,:,2),settings);
     %========Motion estimation===================
     [U,cf] = motion_estimation(I,a,D,U,settings);
     U(isnan(U)) = 0;       
     %========Compute change======================
     inner_c(inner_iter+1) = cf.total; 
     inner_e = abs(inner_c(inner_iter+1)-inner_c(inner_iter))/inner_c(inner_iter);
     inner_iter = inner_iter +1;
     toc; 
     %========Plot current U======================
     if settings.plot_motion
        Uf = U(1:settings.image_size0(1),1:settings.image_size0(2),:); 
        figure(1), 
        subplot(211), imagesc([Ub(:,:,i1,1).*settings.mask(:,:,i1) Uf(:,:,1).*settings.mask(:,:,i1)]);xlabel('Groundtruth vs estimation (horizontal)'),colorbar; 
        subplot(212), imagesc([Ub(:,:,i1,2).*settings.mask(:,:,i1) Uf(:,:,2).*settings.mask(:,:,i1)]);xlabel('Groundtruth vs estimation (vertical)'),colorbar;drawnow;
     end
     end    
              
Uf = U(1:settings.image_size0(1),1:settings.image_size0(2),:); 
outer_iter = outer_iter+1; 
end

%========Compute error======================
error.map  = sqrt((Uf(:,:,1)-Ub(:,:,i1,1)).^2+(Uf(:,:,2)-Ub(:,:,i1,2)).^2); 
error.mean = mean(error.map(settings.mask(:,:,i1)==1));
error.std  = std(error.map(settings.mask(:,:,i1)==1));
disp(['-->  Error = ' num2str(error.mean) ' +- ' num2str(error.std)]); 

%========Plot final U and error=============
figure(2),
subplot(131), imagesc(sqrt(Ub(:,:,i1,1).^2+Ub(:,:,i1,2).^2).*settings.mask(:,:,i1));xlabel('Groundtruth'),colorbar;caxis([0 2])
subplot(132), imagesc(sqrt(Uf(:,:,1).^2+Uf(:,:,2).^2).*settings.mask(:,:,i1));xlabel('Estimation'),colorbar;caxis([0 2])
subplot(133), imagesc(error.map.*settings.mask(:,:,i1));xlabel('Error'),colorbar; 
title('Motion end error maps') 

