function [U, cf] = motion_estimation(images,a,D,U0,settings)
%% Initialization and parameters
% Optimization settings:
optim.itermax = 100;                                              
optim.emax    = 1e-6;                                            
optim.gamma   = 0.01;                                                      % step size                                            
optim.anneal  = 0.8;

% Initialization
[gradx, grady]  = gradient(images(:,:,2));
im    = cat(3,images(:,:,2),gradx,grady);
[x,y] = meshgrid(1:settings.image_size(2),1:settings.image_size(1));
Xgrid = cat(3,x,y);
X     = Xgrid + U0;                                      
X(isnan(X)) = 0;
settings.K = mirt2D_initK(settings.image_size);

% Compute the initial objective function and its gradient
% initial data term 
im_int = mirt2D_mexinterp(im,X(:,:,1),X(:,:,2));
fe     = (im_int(:,:,1)-images(:,:,1))/settings.alpha;
dd     = 2*tanh(fe);
fe     = 2*nansum(log(cosh(fe(:))));
ddx    = dd.*im_int(:,:,2); ddx(isnan(ddx))=0;
ddy    = dd.*im_int(:,:,3); ddy(isnan(ddy))=0;
optim.gamma  = optim.gamma/std([ddx(:); ddy(:)],0);

% initial patch term 
Da(:,:,1) = D(:,:,1)*a(:,:,1);
Da(:,:,2) = D(:,:,2)*a(:,:,2);
PU  = im2patch_2(U0, settings);
E   = PU - Da;
fd = 0.5*settings.lambda_d*nansum(E(:).^2);
g0  = patch2im_2(E,settings);
g0  = settings.lambda_d*g0;

% initial regularization term
dux = mirt_dctn(X(:,:,1));
duy = mirt_dctn(X(:,:,2));
fr  = 0.5*settings.lambda_s*nansum(settings.K(:).*(dux(:).^2+duy(:).^2));   

% initial objective function
T = cat(3,ddx,ddy)+g0;
f = fe+fd+fr;
fchange = optim.emax+1;                                                    
iter    = 0;

%% Motion estimation 
while (abs(fchange)>optim.emax) && (iter<optim.itermax)
    
    % new positions Xp
    Xp = X-Xgrid; 
    Xp = Xp-T*optim.gamma;
    Xp(isnan(Xp)) = 0;
    diax = settings.lambda_s*optim.gamma*settings.K+1;
    dux  = mirt_dctn(Xp(:,:,1))./diax;
    duy  = mirt_dctn(Xp(:,:,2))./diax;
    fr   = 0.5*settings.lambda_s*nansum(settings.K(:).*(dux(:).^2+duy(:).^2));                     
    dux  = mirt_idctn(dux);
    duy  = mirt_idctn(duy);
    Xp   = cat(3,dux,duy)+Xgrid;
     
    % data term
    im_int = mirt2D_mexinterp(im,Xp(:,:,1),Xp(:,:,2));
    fe     = (im_int(:,:,1)-images(:,:,1))/settings.alpha;
    dd     = 2*tanh(fe);
    fe     = 2*nansum(log(cosh(fe(:))));
    ddp(:,:,1)   = dd.*im_int(:,:,2); 
    ddp(:,:,2)   = dd.*im_int(:,:,3); 
    ddp(isnan(ddp))=0;

    % patch term
    U   = Xp - Xgrid;
    Da(:,:,1) = D(:,:,1)*a(:,:,1);
    Da(:,:,2) = D(:,:,2)*a(:,:,2);
    PU  = im2patch_2(U,settings); 
    E   = PU - Da; 
    fd = 0.5*settings.lambda_d*nansum(E(:).^2); 
    gp = patch2im_2(E,settings); 
    gp = settings.lambda_d*gp;
    
    % relative objective function change    
    Tp = gp + ddp;
    fp = fe+fd+fr;
    fchange = (fp-f)/f;
    
    % step size
    if ((fp-f)>0)
        optim.gamma = optim.gamma*optim.anneal;
        f           = f+0.001*abs(f);
    else
        X = Xp; f = fp; T = Tp;
    end
    % cost
    cf.data    = fe;
    cf.spatial = fr;
    cf.sparse  = fd;
    cf.total   = fp;
    iter       = iter+1;
end

% recover 2D motion fields
U = X-Xgrid;

end

